// APISpy.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "../Common/APISpy.h"

#ifdef _MANAGED
#pragma managed(push, off)
#endif

//
// The real copy/move/delete file routines
//
static BOOL ( __stdcall * RealCopyFileExW )(
	LPCWSTR				lpExistingFileName,
	LPCWSTR				lpNewFileName,
	LPPROGRESS_ROUTINE	lpProgressRoutine,
	LPVOID				lpData,
	LPBOOL				pbCancel,
	DWORD				dwCopyFlags ) = CopyFileExW;

static BOOL ( __stdcall * RealMoveFileExW )( 
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName,
    DWORD dwFlags
    ) = MoveFileExW;

static BOOL ( __stdcall * RealMoveFileWithProgressW )(
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName,
    LPPROGRESS_ROUTINE lpProgressRoutine,
    LPVOID lpData,
    DWORD dwFlags
	) = MoveFileWithProgressW;

static BOOL ( __stdcall * RealDeleteFileW )(
    LPCWSTR lpFileName
    ) = DeleteFileW;

void TraceCopyMoveFile( HWND hwndTarget,
					    APISpyOPCodes opcode,
						LPCWSTR	lpExistingFileName,
					    LPCWSTR lpNewFileName )
{
	if( hwndTarget == NULL )
		return;

	//
	// package the data
	//
	API_SPY_TRACE trace;
	trace.opcode = opcode;
	trace.pid = GetCurrentProcessId();

	//
	// marshal it into a byte stream
	//
	std::stringstream stream( ios_base::out | ios_base::binary );
	stream.write( (const char *)&trace, sizeof( API_SPY_TRACE ) );

	//
	// write the from and to file path
	//
	if( lpExistingFileName != NULL )
	{
		std::streamsize len = (std::streamsize)wcslen( lpExistingFileName );
		stream<<len;
		stream.write( (const char *)lpExistingFileName, sizeof( WCHAR ) * len );
	}

	if( lpNewFileName != NULL )
	{
		std::streamsize len = (std::streamsize)wcslen( lpNewFileName );
		stream<<len;
		stream.write( (const char *)lpNewFileName, sizeof( WCHAR ) * len );
	}

	//
	// send this data to the target window
	//
	COPYDATASTRUCT copy;

	string buffer = stream.str();
	copy.cbData = (DWORD)buffer.length();
	copy.dwData = 1;
	copy.lpData = (PVOID)buffer.c_str();
	SendMessage( hwndTarget, WM_COPYDATA, NULL, (LPARAM)&copy );
}

//
// Our proxy copy file routine
//
BOOL __stdcall ProxyCopyFileExW(
	LPCWSTR				lpExistingFileName,
	LPCWSTR				lpNewFileName,
	LPPROGRESS_ROUTINE	lpProgressRoutine,
	LPVOID				lpData,
	LPBOOL				pbCancel,
	DWORD				dwCopyFlags )
{
	//
	// get a handle to the target window
	//
	HWND hwndTarget = FindWindowEx( HWND_MESSAGE, NULL, API_SPY_TRACE_CLASS_NAME, API_SPY_TRACE_CLASS_NAME );
	if( hwndTarget != NULL )
		TraceCopyMoveFile( hwndTarget, opCopyFile, lpExistingFileName, lpNewFileName );

	//
	// invoke the original routine
	//
	return RealCopyFileExW(
		lpExistingFileName,
		lpNewFileName,
		lpProgressRoutine,
		lpData,
		pbCancel,
		dwCopyFlags );
}

//
// Our proxy move file routine
//
BOOL __stdcall ProxyMoveFileExW(
	LPCWSTR				lpExistingFileName,
	LPCWSTR				lpNewFileName,
	DWORD				dwFlags )
{
	//
	// get a handle to the target window
	//
	HWND hwndTarget = FindWindowEx( HWND_MESSAGE, NULL, API_SPY_TRACE_CLASS_NAME, API_SPY_TRACE_CLASS_NAME );
	if( hwndTarget != NULL )
		TraceCopyMoveFile( hwndTarget, opMoveFile, lpExistingFileName, lpNewFileName );

	//
	// invoke the original routine
	//
	return RealMoveFileExW(
		lpExistingFileName,
		lpNewFileName,
		dwFlags );
}

BOOL __stdcall ProxyMoveFileWithProgressW(
    LPCWSTR lpExistingFileName,
    LPCWSTR lpNewFileName,
    LPPROGRESS_ROUTINE lpProgressRoutine,
    LPVOID lpData,
    DWORD dwFlags )
{
	//
	// get a handle to the target window
	//
	HWND hwndTarget = FindWindowEx( HWND_MESSAGE, NULL, API_SPY_TRACE_CLASS_NAME, API_SPY_TRACE_CLASS_NAME );
	if( hwndTarget != NULL )
		TraceCopyMoveFile( hwndTarget, opMoveFile, lpExistingFileName, lpNewFileName );

	//
	// invoke the original routine
	//
	return RealMoveFileWithProgressW(
		lpExistingFileName,
		lpNewFileName,
		lpProgressRoutine,
		lpData,
		dwFlags );
}

BOOL __stdcall ProxyDeleteFileW( LPCWSTR lpFileName )
{
	//
	// get a handle to the target window
	//
	HWND hwndTarget = FindWindowEx( HWND_MESSAGE, NULL, API_SPY_TRACE_CLASS_NAME, API_SPY_TRACE_CLASS_NAME );
	if( hwndTarget != NULL )
		TraceCopyMoveFile( hwndTarget, opDeleteFile, lpFileName, NULL );

	//
	// invoke the original routine
	//
	return RealDeleteFileW( lpFileName );
}

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  dwReason,
                       LPVOID lpReserved
					 )
{
	if( dwReason == DLL_PROCESS_ATTACH )
	{
		DetourTransactionBegin();
        DetourUpdateThread( GetCurrentThread() );
        DetourAttach( &(PVOID&)RealCopyFileExW, ProxyCopyFileExW );
		DetourAttach( &(PVOID&)RealMoveFileExW, ProxyMoveFileExW );
		DetourAttach( &(PVOID&)RealMoveFileWithProgressW, ProxyMoveFileWithProgressW );
		DetourAttach( &(PVOID&)RealDeleteFileW, ProxyDeleteFileW );
        DetourTransactionCommit();
	}
	else if( dwReason == DLL_PROCESS_DETACH )
	{
		DetourTransactionBegin();
        DetourUpdateThread( GetCurrentThread() );
        DetourDetach( &(PVOID&)RealCopyFileExW, ProxyCopyFileExW );
		DetourDetach( &(PVOID&)RealMoveFileExW, ProxyMoveFileExW );
		DetourDetach( &(PVOID&)RealMoveFileWithProgressW, ProxyMoveFileWithProgressW );
		DetourDetach( &(PVOID&)RealDeleteFileW, ProxyDeleteFileW );
        DetourTransactionCommit();
	}

    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif
