#include "StdAfx.h"
#include "ToolHelp.h"

bool CToolHelp::GetProcessList32( vector<PROCESSENTRY32>& processes )
{
	//
	// get a snap of all running processes
	//
	HANDLE hSnapshot = CreateToolhelp32Snapshot( TH32CS_SNAPPROCESS, 0 );
	if( hSnapshot == INVALID_HANDLE_VALUE )
		return false;

	//
	// get the first entry
	//
	PROCESSENTRY32 entry;
	entry.dwSize = sizeof( PROCESSENTRY32 );
	if( Process32First( hSnapshot, &entry ) == FALSE )
	{
		CloseHandle( hSnapshot );
		return false;
	}

	//
	// add it to the list and get the rest of the entries
	// excluding the current process
	//
	processes.clear();
	DWORD dwCurrentPID = GetCurrentProcessId();

	do
	{
		if( dwCurrentPID != entry.th32ProcessID )
			processes.push_back( entry );

	} while( Process32Next( hSnapshot, &entry ) == TRUE );

	CloseHandle( hSnapshot );
	return true;
}

bool CToolHelp::LoadLibraryRemote( DWORD dwPID, const TCHAR *pszPath )
{
	USES_CONVERSION;

	//
	//
	// open a handle to the process
	//
	HANDLE hProcess = OpenProcess( PROCESS_CREATE_THREAD | PROCESS_QUERY_INFORMATION | PROCESS_VM_OPERATION | PROCESS_VM_WRITE | PROCESS_VM_READ,
									FALSE, dwPID );
	if( hProcess == NULL )
		return false;

	HMODULE hKernel32 = LoadLibrary( _T( "KERNEL32.DLL" ) );
	if( hKernel32 == NULL )
	{
		CloseHandle( hProcess );
		return false;
	}

	//
	// get the path to the apispy.dll file
	//
	char szFilePath[MAX_PATH];
	GetModuleFileNameA( NULL, szFilePath, MAX_PATH );
	PathRemoveFileSpecA( szFilePath );
	PathAppendA( szFilePath, T2CA( pszPath ) );

	//
	// allocate space for storing this path in the remote
	// process
	//
	void *pFilePath = VirtualAllocEx( hProcess, NULL, sizeof( szFilePath ), MEM_COMMIT, PAGE_READWRITE );
	if( pFilePath == NULL )
	{
		CloseHandle( hProcess );
		FreeLibrary( hKernel32 );
		return false;
	}

	//
	// write the path in the remote process
	//
	SIZE_T cbWritten;
	if( WriteProcessMemory( hProcess, pFilePath, szFilePath, strlen( szFilePath ), &cbWritten ) == FALSE )
	{
		CloseHandle( hProcess );
		FreeLibrary( hKernel32 );
		VirtualFreeEx( hProcess, pFilePath, sizeof( szFilePath ), MEM_RELEASE );
		return false;
	}

	//
	// load our DLL in the remote process
	//
	DWORD dwThreadID;
	HANDLE hRemoteThread = CreateRemoteThread( hProcess, NULL, 0,
											   (LPTHREAD_START_ROUTINE)GetProcAddress( hKernel32, _T( "LoadLibraryA" ) ),
											   pFilePath, 0, &dwThreadID );
	WaitForSingleObject( hRemoteThread, INFINITE );

	CloseHandle( hRemoteThread );
	CloseHandle( hProcess );
	FreeLibrary( hKernel32 );

	return true;
}
